/* Deep Waters v0.5.0 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compose = require('./compose.js');
var isFunction = require('./isFunction.js');
var every = require('./every.js');
var isNumber = require('./isNumber.js');
var isString = require('./isString.js');
var when = require('./when.js');
var or = require('./or.js');
var isBoolean = require('./isBoolean.js');
var isNull = require('./isNull.js');
var isObject = require('./isObject.js');
var isUndefined = require('./isUndefined.js');
var isRegExp = require('./isRegExp.js');
var isDate = require('./isDate.js');

var objectEquals = function objectEquals(object) {
  return function (value) {
    if (!isObject.default(object)) return false;
    var objKeys = Object.keys(object);
    var valueKeys = Object.keys(value);
    if (objKeys.length !== valueKeys.length) return false; // eslint-disable-next-line no-use-before-define

    return valueKeys.every(function (key) {
      return equals(value[key])(object[key]);
    });
  };
}; // eslint-disable-next-line no-use-before-define


var arrayEquals = function arrayEquals(array) {
  return every.default(function (item, index) {
    return equals(item)(array[index]);
  });
};

var equals = function equals(value) {
  return compose.default(when.default(or.default(isString.default, isBoolean.default, isNumber.default, isFunction.default, isRegExp.default, isNull.default, isUndefined.default), function (string) {
    return value === string;
  }), when.default(isDate.default, function () {
    return !!value.valueOf();
  }), when.default(Array.isArray, arrayEquals(value)), when.default(isObject.default, objectEquals(value)));
};

exports.default = equals;
